/** 定义存储整个数组的区域划分的数据 */
var list = [];

/** 储存状态 - 是否为编辑模式 */
var editStatus = false;

/** 存储地图 */
var map;

/** 根据状态显示按钮 */
var btnStatusMethod = function () {
    /** 获取开始编辑按钮 */
    var startEdit = document.getElementById('startEdit'),
        /** 获取结束保存按钮 */
        endEdit = document.getElementById('endEdit');
    /** 根据状态选显示不同按钮 */
    if (editStatus) {
        endEdit.style.display = 'block';
        startEdit.style.display = 'none';
        return;
    }
    startEdit.style.display = 'block';
    endEdit.style.display = 'none';
    return;
}

/** 生成 (范围性) 随机数 [PS: 用于生成随机颜色码]*/
var randomNum = function (minNum, maxNum) {
    switch (arguments.length) {
        case 1:
            return parseInt(Math.random() * minNum + 1, 10);
        case 2:
            return parseInt(Math.random() * (maxNum - minNum + 1) + minNum, 10);
        default:
            return 0;
    }
}

/** 验证是否为数字类型字符串 - 非数字类型直接替换成空 */
var NumberCheck = function (str) {
    var len1 = str.substr(0, 1);
    var len2 = str.substr(1, 1);
    //如果第一位是0，第二位不是点，就用数字把点替换掉
    if (str.length > 1 && len1 == 0 && len2 != ".") {
        str = str.substr(1, 1);
    }
    //第一位不能是.
    if (len1 == ".") {
        str = "";
    }
    //限制只能输入一个小数点
    if (str.indexOf(".") != -1) {
        var str_ = str.substr(str.indexOf(".") + 1);
        if (str_.indexOf(".") != -1) {
            str = str.substr(0, str.indexOf(".") + str_.indexOf(".") + 1);
        }
    }
    //正则替换，保留数字和小数点
    str = str.replace(/[^\d^\.]+/g, '');
    return str;
}

/**
 * 根据经纬度计算四个坐标的经纬度
 * @param {Float} lng 经度
 * @param {Float} lat 纬度
 * @returns {Array} 返回四个角经纬度
 */
var computeLngLat = function (lng, lat) {
    /** 地球半径千米 */
    var r = 6371;
    /** 0.5千米距离 */
    var dis = 0.5;
    /** 角度转为弧度 */
    var dlng = 2 * Math.asin(Math.sin(dis / (2 * r)) / Math.cos(lat * Math.PI / 180));
    dlng = dlng * 180 / Math.PI;
    var dlat = dis / r;
    dlat = dlat * 180 / Math.PI;
    return [
        /** 左上角 */
        [lng - dlng, lat + dlat],
        /** 左下角 */
        [lng - dlng, lat - dlat],
        /** 右下角 */
        [lng + dlng, lat - dlat],
        /** 右上角 */
        [lng + dlng, lat + dlat],
    ];
};

/** 创建并且插入每个单独的dom */
var createDomListItem = function () {
    /** 通过动态参数存储 color 值 */
    var color = arguments[arguments.length - 1];
    /** 初始化data - 以免没有传递参数报错 */
    var data = {
        /** 区域名称 */
        regionName: '',
        /** 起送价 */
        startPrice: '',
        /** 配送费 */
        deliveryPrice: '',
    };
    /** 判断参数的长度是否大于 1，如果大于则代表第一个参数 data存在 */
    if (arguments.length > 1) {
        /** 覆盖默认的参数 */
        data = arguments[0]
    }
    /** 获取 ul 列表dom */
    var listDynamic = document.getElementsByClassName('listDynamic')[0];
    /** 创建 item 容器 */
    var itemDom = document.createElement('li');
    /** 写入html内容进 item */
    itemDom.innerHTML = '<div class="top clear">' +
        '<i class="iconColor" style="background-color:rgba(' + color + ',.5);border:1px solid rgb(' + color + ')"></i>' +
        '<input type="text" placeholder="请输入区域名称" name="regionName" value="' + data.regionName + '">' +
        '<i class="iconDelete"></i>' +
        '</div>' +
        '<div class="price clear">' +
        '<strong>起送价：</strong>' +
        '<div class="priceContainer clear">' +
        '<input type="text" name="startPrice" placeholder="请输入" value="' + data.startPrice + '">' +
        '</div>' +
        '</div>' +
        '<div class="price clear">' +
        '<strong>配送费：</strong>' +
        '<div class="priceContainer clear">' +
        '<input type="text" name="deliveryPrice" placeholder="请输入" value="' + data.deliveryPrice + '">' +
        '</div>' +
        '</div>';
    /** 插入dom */
    listDynamic.appendChild(itemDom);
    /** 让列表始终滚到最底部 */
    listDynamic.scrollTop = listDynamic.scrollHeight + 20;
    /** 获取所有 item */
    var item = listDynamic.getElementsByTagName('li');
    /** 清楚所有 item 的 class */
    var clearItemClass = function () {
        /** 循环所有item - 清楚所有 item 的 class */
        for (var i = 0; i < item.length; i++) {
            /** 闭包写法 */
            (function (x) {
                /** 判断 item 是否有 class 存在 */
                if (item[x].getAttribute('class')) {
                    /** 清楚 class */
                    item[x].removeAttribute('class');
                }
            })(i);
        }
    }
    /** 循环所有item并添加点击事件 */
    for (var i = 0; i < item.length; i++) {
        /** 作用域 - 防止闭包自执行函数 */
        (function (x) {
            /** item 点击事件 */
            item[x].onclick = function () {
                /** 清楚所有item的 class */
                clearItemClass();
                /** 给当前class 添加选中 active 状态 */
                this.setAttribute('class', 'active');
            };
            /** 每个 item 的删除按钮点击事件 */
            item[x].getElementsByClassName('iconDelete')[0].onclick = function () {
                /** 删除地图上已有的多边形区域 */
                list[x].polygon.setMap(null);
                /** 关闭当前编辑的图层 */
                list[x].polyEditor.close();
                /** 删除当前列表中的 item */
                listDynamic.removeChild(item[x]);
                /** 将 list 数组中已绑定的数据 */
                list.splice(x, 1);
                /** 判断是否为最后一个区域 */
                if (list.length === 0) {
                    /** 如果只剩下最后一个区域，那就关闭编辑状态 */
                    editStatus = false;
                    /** 根据状态显示按钮 */
                    btnStatusMethod();
                }
            };
            /** 获取所有 input */
            var input = item[x].getElementsByTagName('input');
            /** 循环所有 input 并且添加相应事件 */
            for (var j = 0; j < input.length; j++) {
                /** 闭包写法 - 防止出现 bug */
                (function (y) {
                    /** input 输出触发事件 */
                    input[y].oninput = function () {
                        /** input 的 name */
                        var name = this.getAttribute('name');
                        /** input 的 value */
                        var value = this.value;
                        /** 区域名称 */
                        if (name === 'regionName') {
                            /** 将输入的区域名称赋值进 list 数组的 data.regionName 中 */
                            list[x].data.regionName = value;
                        }
                        /** 统一判断金额验证 */
                        else {
                            /** 验证金额正则 */
                            var regularMoney = /^\d+\.?\d{0,2}$/;
                            /** 判断是否为金额类型的数字或者为输入 */
                            if (regularMoney.test(value) || value === '') {
                                /** 起送价 */
                                if (name === 'startPrice') {
                                    /** 将输入的起送价赋值进 list 数组的 data.startPrice 中 */
                                    list[x].data.startPrice = value;
                                }
                                /** 配送费 */
                                else if (name === 'deliveryPrice') {
                                    /** 将输入的配送费赋值进 list 数组的 data.deliveryPrice 中 */
                                    list[x].data.deliveryPrice = value;
                                }
                            }
                            /** 类型错误 */
                            else {
                                this.value = Number(NumberCheck(value)).toFixed(2);
                                /** 起送价 */
                                if (name === 'startPrice') {
                                    /** 将输入的起送价赋值进 list 数组的 data.startPrice 中 */
                                    list[x].data.startPrice = this.value;
                                }
                                /** 配送费 */
                                else if (name === 'deliveryPrice') {
                                    /** 将输入的配送费赋值进 list 数组的 data.deliveryPrice 中 */
                                    list[x].data.deliveryPrice = this.value;
                                }
                            }
                        }
                    }
                })(j);
            }
        })(i);
    };
};

/** 向地图添加拖拽矩形 */
var addMapDragRectangle = function () {
    /** 获取最后一个参数，并且赋值 [该参数为经纬度数组] */
    var path = arguments[arguments.length - 1];
    var defaultData = {
        /** 区域名称 */
        regionName: '',
        /** 起送价 */
        startPrice: '',
        /** 配送费 */
        deliveryPrice: '',
    };
     /** 判断参数的长度是否大于 1，如果大于则代表第一个参数 data 存在 */
     if (arguments.length > 1) {
        /** 覆盖默认的参数 */
        defaultData = arguments[0]
    }
    /** 随机生成颜色 */
    var color = [randomNum(0, 255), randomNum(0, 255), randomNum(0, 255)];
    /** 创建地图的矩形对象 */
    var polygon = new AMap.Polygon({
        // 多边形路径点(根据经纬度设置多个点)
        path: path,
        // 多边形描边透明度
        strokeOpacity: 0,
        // 多边形透明度
        fillOpacity: 1,
        // 多边形颜色
        fillColor: 'rgba(' + color.toString() + ',.5)',
        // 多边形层级
        zIndex: 50,
    });
    /** 创建工具类 - 编辑矩形类 */
    var polyEditor = new AMap.PolyEditor(map, polygon);
    /** 判断是否已经开启类编辑模式 */
    if (editStatus) {
        polyEditor.open()
    }
    /** 向数组添加数据集合 */
    list.push({
        /** 存放数据的对象 */
        data: Object.assign(defaultData,{
            /** 经纬度数组存放 */
            lngLatArr: path,
        }),
        /** 地图的矩形对象 */
        polygon: polygon,
        /** 工具类 - 编辑矩形类 */
        polyEditor: polyEditor,
        /** 随机生成的颜色值 */
        color: color.toString(),
    });
    // 将多边形添加到地图中
    map.add(list[list.length - 1].polygon);
    // 缩放地图到合适的视野级别
    map.setFitView(list[list.length - 1].polygon);
    /** 生成并且插入 dom */
    createDomListItem(defaultData, color.toString());
    /** 编辑按钮点击事件 */
    document.getElementById('startEdit').onclick = function () {
        if (list.length !== 0) {
            /** 循环 list 列表，并且处理事件 */
            for (var i = 0; i < list.length; i++) {
                /** 作用域闭包写法 */
                (function (x) {
                    /** 让所有模块开始编辑 */
                    list[x].polyEditor.open();
                })(i);
            }
        }
        /** 将状态改编成修改模式 */
        editStatus = true;
        /** 根据状态显示按钮 */
        btnStatusMethod();
    };
    /** 结束编辑事件 */
    document.getElementById('endEdit').onclick = function () {
        /** 循环 list 列表，并且处理事件 */
        for (var i = 0; i < list.length; i++) {
            /** 作用域闭包写法 */
            (function (x) {
                /** 让所有模块开始编辑 */
                list[x].polyEditor.close();
            })(i);
        }
        /** 存放后端需要的数据 */
        var data = [];
        /** 循环 list */
        list.forEach(function (item) {
            /** 将后端有用的数据提取出来放到 data 数组中 */
            data.push(item.data);
        });
        console.log(JSON.stringify(data))
        /** 将状态调整为 编辑完成 */
        editStatus = false;
        /** 根据状态显示按钮 */
        btnStatusMethod();
        /** 调用 http 方法 */
        httpMethod(data);
    };
};

/** 新增配送区域按钮 - 点击事件 */
document.getElementById('addLayer').onclick = function () {
    /** 获取地图中心经度 */
    var lng = map.Qe.center.lng,
        /** 获取地图中心纬度 */
        lat = map.Qe.center.lat,
        /** 求出四个角经纬度 */
        arrLngLat = computeLngLat(lng, lat);
        console.log(arrLngLat);
    /** 向地图插入一个矩形 */
    addMapDragRectangle(arrLngLat);
};

/** 初始化执行函数 */
var initialize = function () {
    /** 初始化地图 */
    map = new AMap.Map("container", {
        /** 自动定位当前城市 */
        resizeEnable: true,
        /** 地图缩放层级 */
        zoom: 12,
    });
    /** 根据状态显示按钮 */
    btnStatusMethod();
    /** 循环初始数据 */
    for(var i = 0; i < data.length; i++) {
        /** 闭包写法 */
        (function(x){
            /** 组合左侧显示数据 */
            var obj = {
                regionName: data[x].regionName,
                startPrice: data[x].startPrice,
                deliveryPrice: data[x].deliveryPrice,
            };
            /** 存储经纬度 */
            var lngLatArr = [];
            /** 循环经纬度 */
            data[x].lngLatArr.forEach(function(item) {
                /** 数据转换类型并添加进 lngLatArr 数组 */
                lngLatArr.push([item.lng, item.lat])
            })
            /** 添加矩形 */
            addMapDragRectangle(obj,lngLatArr);
        })(i);
    };
};

/** 立即执行初始化函数 */
initialize();